@@ -186,6 +186,22 @@ string CyclicCoordinateDescent::getCrossValidationInfo() const {
     return crossValidationInfo;
 }
 
+// void CyclicCoordinateDescent::setBootStrapInfo(string info) {
+//     bootStrapInfo = info;
+// }
+//
+// string CyclicCoordinateDescent::getBootStrapInfo() const {
+//     return bootStrapInfo;
+// }
+
+void CyclicCoordinateDescent::setBootStrapInfo(const std::vector<double>& info) {
+    bootStrapInfoVector = info;
+}
+
+const std::vector<double>& CyclicCoordinateDescent::getBootStrapInfo() const {
+    return bootStrapInfoVector;
+}
+
 void CyclicCoordinateDescent::setPrior(priors::JointPriorPtr newPrior) {
     jointPrior = newPrior;
 }
@@ -1085,7 +1101,11 @@ void CyclicCoordinateDescent::findMode(
 		//modelSpecifics.resetBeta();
 	}
 
+// <<<<<<< HEAD
 	auto cycle = [this,&iteration,algorithmType,&allDelta,&doItAll] {
+// =======
+// 	auto cycle = [this,&lastObjFunc,&lastObjFuncVec,&iteration,algorithmType,&allDelta,&doItAll] {
+// >>>>>>> jni
 /*
 		if (iteration%10==0) {
 			std::cout<<"iteration " << iteration << " ";
@@ -1381,6 +1401,30 @@ void CyclicCoordinateDescent::findMode(
  * Computationally heavy functions
  */
 
+double CyclicCoordinateDescent::getLogPriorGradient(int index) {
+    double gradient = 0.0;
+
+
+    if (jointPrior->getPriorType(index) != priors::PriorType::NONE) {
+        std::ostringstream stream;
+        stream << "getLogPriorGradient is not yet implemented for non-flat priors";
+        error->throwError(stream);
+    }
+
+    return gradient;
+}
+
+double CyclicCoordinateDescent::getLogLikelihoodGradient(int index) {
+
+ 	checkAllLazyFlags();
+
+ 	double gradient, hessian;
+ 	computeNumeratorForGradient(index);
+ 	modelSpecifics.computeGradientAndHessian(index, &gradient, &hessian, useCrossValidation);
+
+ 	return -gradient;
+ }
+
 void CyclicCoordinateDescent::computeGradientAndHessian(int index, double *ogradient,
 		double *ohessian) {
 	modelSpecifics.computeGradientAndHessian(index, ogradient, ohessian, useCrossValidation);
