@@ -75,6 +75,10 @@ public:
 
 	double getLogLikelihood(void);
 
+	double getLogLikelihoodGradient(int index);
+
+	double getLogPriorGradient(int index);
+
 	//double getPredictiveLogLikelihood(double* weights);
 
 	double getNewPredictiveLogLikelihood(double* weights);
@@ -158,6 +162,14 @@ public:
 
 	void setCrossValidationInfo(string info);
 
+	// string getBootStrapInfo() const;
+	//
+	// void setBootStrapInfo(string info);
+
+	const std::vector<double>& getBootStrapInfo() const;
+
+	void setBootStrapInfo(const std::vector<double>& info);
+
 	string getConditionId() const {
 		return conditionId;
 	}
@@ -427,6 +439,8 @@ protected:
 	SetBetaContainer setBetaList;
 
 	string crossValidationInfo;
+	string bootStrapInfo;
+	std::vector<double> bootStrapInfoVector;
 
 	loggers::ProgressLoggerPtr logger;
 	loggers::ErrorHandlerPtr error;
