@@ -29,9 +29,9 @@ public:
 
 	virtual void permute();
 
-	virtual void getWeights(int batch, std::vector<real>& weights);
+	virtual void getWeights(int batch, std::vector<double>& weights);
 
-	virtual void getComplement(std::vector<real>& weights);
+	virtual void getComplement(std::vector<double>& weights);
 	
 	AbstractSelector* clone() const;
 
