@@ -43,6 +43,8 @@ public:
 
 	virtual double getKktBoundary(const int index) const = 0; // pure virtual
 
+	virtual PriorType getPriorType(const int index) {return PriorType::NONE;};
+
 //  	virtual JointPrior* clone() const = 0; // pure virtual
 
     void addVarianceParameter(const VariancePtr& ptr) {
@@ -102,6 +104,10 @@ public:
 		return stream.str();
 	}
 
+	PriorType getPriorType(int index) {
+		return listPriors[index]->getPriorType();
+	}
+
 // 	void setVariance(int index, double x) {
 // // 		for (PriorList::iterator it = uniquePriors.begin(); it != uniquePriors.end(); ++it) {
 // // 			(*it)->setVariance(level, x);
@@ -637,6 +643,10 @@ public:
 		return singlePrior->getKktBoundary();
 	}
 
+	PriorType getPriorType(int index) {
+		return singlePrior->getPriorType();
+	}
+
 // 	JointPrior* clone() const {
 // 	    std::vector<VariancePtr> newPtrs;
 // 	    for (auto x : variance) {
